/*
    kopeteviewmanager.cpp - View Manager

    Copyright (c) 2003 by Jason Keirstead
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <knotifyclient.h>
#include <kwin.h>

#include "kopeteprefs.h"
#include "kopeteprotocol.h"
#include "kopetemessagemanager.h"
#include "kopetechatwindow.h"
#include "kopeteemailwindow.h"
#include "kopeteevent.h"
#include "chatview.h"

#include "kopeteviewmanager.h"

KopeteView::KopeteView( KopeteMessageManager *manager )
{
	m_manager = manager;
}

KopeteViewManager *KopeteViewManager::s_viewManager = 0L;

KopeteViewManager *KopeteViewManager::viewManager()
{
	if( !s_viewManager )
		s_viewManager = new KopeteViewManager;

	return s_viewManager;
}

KopeteViewManager::KopeteViewManager() : QObject( kapp, "KopeteViewManager" )
{
	m_activeView = 0L;
	connect( KopetePrefs::prefs(), SIGNAL( saved() ), this, SLOT( slotPrefsChanged() ) );
	slotPrefsChanged();
}

KopeteViewManager::~KopeteViewManager()
{
}

void KopeteViewManager::slotPrefsChanged()
{
	useQueue = KopetePrefs::prefs()->useQueue();
	raiseWindow = KopetePrefs::prefs()->raiseMsgWindow();
}

KopeteView *KopeteViewManager::view( KopeteMessageManager* manager, bool foreignMessage, KopeteView::ViewType type )
{
	if( m_managerMap.contains( manager ) && m_managerMap[ manager ] )
		return m_managerMap[ manager ];
	else
	{
		KopeteView *newView;
		QObject *newObject;

		if( type == KopeteView::Chat )
		{
			KopeteChatWindow *myWindow = findWindow( manager->protocol() );
			newView = static_cast<KopeteView*>( myWindow->addChatView( manager ) );
			newObject = dynamic_cast<QObject*>(newView);

			connect (manager, SIGNAL(contactAdded(const KopeteContact *)), newObject, SLOT(contactAdded(const KopeteContact *)));
			connect (manager, SIGNAL(contactRemoved(const KopeteContact *)), newObject, SLOT(contactRemoved(const KopeteContact *)));
			connect (newObject, SIGNAL( typing(bool) ), manager, SLOT( typing(bool) ) );
			connect (manager, SIGNAL( remoteTyping( const KopeteContact *, bool) ), newObject, SLOT( remoteTyping(const KopeteContact *, bool) ) );
		}
		else
		{
			newView = static_cast<KopeteView*>( new KopeteEmailWindow( manager, foreignMessage ) );
			newObject = dynamic_cast<QObject*>(newView);
		}

		emit( viewCreated( newView ) );

		m_managerMap.insert( manager, newView );

		connect (newObject, SIGNAL( closing( KopeteView * ) ), this, SLOT( slotViewDestroyed( KopeteView * ) ) );
		connect (newObject, SIGNAL( messageSent(KopeteMessage &) ), manager, SLOT( sendMessage(KopeteMessage &) ) );
		connect (newObject, SIGNAL( activated( KopeteView * ) ), this, SLOT( slotViewActivated( KopeteView * ) ) );
		connect (manager, SIGNAL( messageSuccess()), newObject, SLOT(messageSentSuccessfully()) );
		connect (manager, SIGNAL( closing(KopeteMessageManager *) ), this, SLOT(slotMessageManagerDestroyed(KopeteMessageManager*)) );

		return newView;
	}
}

void KopeteViewManager::messageAppended( KopeteMessage &msg, KopeteMessageManager *manager)
{
	kdDebug( 14010 ) << k_funcinfo << endl;

	bool foreignMessage = ( msg.direction() != KopeteMessage::Outbound );

	if( foreignMessage || m_managerMap.contains(manager) )
	{
		view( manager, foreignMessage )->messageReceived( msg );

		if ( useQueue && foreignMessage && !view( manager, foreignMessage )->isVisible()  )
		{
			if (!m_eventMap.contains( manager ))
			{
				//FIXME: currently there are maximum only one event per kmm.
				KopeteEvent *event=new KopeteEvent(msg,manager);
				m_eventMap.insert( manager, event );
				connect(event, SIGNAL(done(KopeteEvent *)), this, SLOT(slotEventDeleted(KopeteEvent *)));
				emit newMessageEvent(event);
			}
		}
		else
		{
			readMessages( manager, foreignMessage );
		}

		//sounds
		if ( KopetePrefs::prefs()->beepNotify() )
			KNotifyClient::beep();

		if ( foreignMessage && KopetePrefs::prefs()->soundNotify() && ( !manager->protocol()->isAway() || KopetePrefs::prefs()->soundIfAway() ) )
		{
			QString msgFrom = QString::null;
			if( msg.from()->metaContact() )
				msgFrom = msg.from()->metaContact()->displayName();
			else
				msgFrom = msg.from()->displayName();

			KNotifyClient::event( QString::fromLatin1( "kopete_incoming"), i18n("Incoming message from %1").arg(msgFrom) );
		}
	}
}

void KopeteViewManager::readMessages( KopeteMessageManager *manager, bool foreignMessage )
{
	kdDebug( 14010 ) << k_funcinfo << endl;
	KopeteView *thisView = view( manager, foreignMessage );

	if( !thisView->isVisible() )
		thisView->makeVisible();

	if( !foreignMessage || raiseWindow )
		thisView->raise();
}

void KopeteViewManager::launchWindow( KopeteMessageManager *manager, KopeteView::ViewType type )
{
	if( m_eventMap.contains( manager ) )
	{
		m_eventMap[ manager ]->deleteLater();
		m_eventMap.remove( manager );
	}

	view( manager, false, type )->raise();
}

void KopeteViewManager::slotEventDeleted( KopeteEvent *event )
{
	for ( EventMap::Iterator it = m_eventMap.begin(); it != m_eventMap.end(); ++it )
	{
		if( it.data() == event )
		{
			KopeteMessageManager *kmm=it.key();
			m_eventMap.remove( kmm );
			readMessages( kmm, false );
			break;
		}
	}
}

void KopeteViewManager::slotViewActivated( KopeteView *view )
{
	kdDebug( 14010 ) << k_funcinfo << endl;
	m_activeView = view;
}

KopeteChatWindow *KopeteViewManager::findWindow( KopeteProtocol *p )
{
	bool windowCreated = false;
	KopeteChatWindow *myWindow;

	switch( KopetePrefs::prefs()->chatWindowPolicy() )
	{
		case NEW_WINDOW: //Open every chat in a new window
			windowCreated = true;
			break;

		case GROUP_BY_PROTOCOL: //Open chats in the same protocol in the same window
			if( m_protocolMap.contains( p ) )
				myWindow = m_protocolMap[ p ];
			else
				windowCreated = true;
			break;

		case GROUP_ALL: //Open all chats in the same window
			if( windows.isEmpty() )
				windowCreated = true;
			else
			{
				int viewCount = 0;
				for ( KopeteChatWindow *thisWindow = windows.first(); thisWindow; thisWindow = windows.next() )
				{
					if( thisWindow->chatViewCount() > viewCount )
					{
						myWindow = thisWindow;
						viewCount = thisWindow->chatViewCount();
					}
				}
			}
			break;
	}

	if( windowCreated )
	{
		myWindow = new KopeteChatWindow();
		connect(myWindow, SIGNAL(closing(KopeteChatWindow*)), this, SLOT(slotChatWindowDestroyed(KopeteChatWindow*)));

		if( !m_protocolMap.contains( p ) )
			m_protocolMap.insert( p, myWindow );
	}

	kdDebug( 14010 ) << k_funcinfo << "Open Windows: " << windows.count() << endl;

	return myWindow;
}

void KopeteViewManager::slotViewDestroyed( KopeteView *closedView )
{
	if( m_managerMap.contains( closedView->msgManager() ) )
	{
		m_managerMap.remove( closedView->msgManager() );
		closedView->msgManager()->setCanBeDeleted( true );
	}
}

void KopeteViewManager::slotMessageManagerDestroyed( KopeteMessageManager *manager )
{
	if( m_managerMap.contains( manager ) )
	{
		m_managerMap[ manager ]->close( true );
		m_managerMap.remove( manager );
	}
}

void KopeteViewManager::slotChatWindowDestroyed( KopeteChatWindow *window )
{
	for ( ProtocolMap::Iterator it = m_protocolMap.begin(); it != m_protocolMap.end(); ++it )
	{
		if( it.data() == window )
			m_protocolMap.remove( it.key() );
	}

	windows.remove( window );

	kdDebug( 14010 ) << k_funcinfo << "Open Windows: " << windows.count() << endl;
}


#include "kopeteviewmanager.moc"
