/*
    kopeteviewmanager.h - View Manager

    Copyright (c) 2003 by Jason Keirstead
    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef KOPETEVIEWMANAGER_H
#define KOPETEVIEWMANAGER_H

#include <qobject.h>
#include <qmap.h>
#include <qptrlist.h>

#define NEW_WINDOW 0
#define GROUP_BY_PROTOCOL 1
#define GROUP_ALL 2

class KopeteMessageManager;
class KopeteMessage;
class KopeteChatWindow;
class KopeteProtocol;
class KopeteContact;
class KopeteEvent;

/**
 * @author Jason Keirstead
 *
 * Abstract parent class for all types of views used for messaging.  At present
 * these may be either KopeteView::Chat or KopeteView::Email.
 *
 */
class KopeteView
{
	public:
		/**
		 * Specifies the type of the view.
		 * May currently be of type
		 * - Chat: Two way chat
		 * - Email: Single shot messaging
		 */
		enum ViewType { Chat, Email };

		KopeteView( KopeteMessageManager *manager );

		/**
		* Returns the message currently in the edit area
		* @return The message
		*/
		virtual KopeteMessage currentMessage() = 0;
		/**
		 * Set the message that the view is currently editing.
		 * @param newMessage The new message to be edited.
		 */
		virtual void setCurrentMessage( const KopeteMessage &newMessage ) = 0;

		/**
		 * @return The KopeteMessageManager that the view is in communication
		 * with.
		 */
		KopeteMessageManager *msgManager() { return m_manager; }
		
		/**
		 * @return The KopeteView::ViewType of the view
		 */
		ViewType viewType() { return m_type; }

		/**
		 * SIGNAL Emitted when the message is sent.
		 */
		virtual void messageSent( KopeteMessage & ) = 0;

		/**
		 * SLOT to inform the view that the message was sent successfully.
		 */
		virtual void messageSentSuccessfully() = 0;

		/** 
		 * SLOT called to pass a received message to the view.
		 */
		virtual void messageReceived( KopeteMessage & ) = 0;

		/**
		 * SIGNAL Emitted when the chat view is shown.
		 * FIXME: IS THIS IN USE ANYWHERE?
		 */
		virtual void shown() = 0;

		/**
		 * SLOT Sends the view's current message.
		 */
		virtual void sendMessage() = 0;

		/**
		 * SIGNAL Emitted when the view is closing.
		 */
		virtual void closing( KopeteView * ) = 0;

		/**
		 * Raises the view to the top of the screen.
		 */
		virtual void raise() = 0;

		/**
		 * Make the view visible if it is currently hidden.
		 */
		virtual void makeVisible() = 0;

		/*
		 * Close this view
		 */
		virtual bool close( bool force = false ) = 0;

		/** 
		 * Is the view currently visible.
		 * @return Whether the view is visible or not.
		 */
		virtual bool isVisible() = 0;

		/**
		 * SIGNAL Emitted when the view is activated ( raised by the user )
		 */
		virtual void activated( KopeteView * ) = 0;

		/**
		 * Returns the widget used to compose messages
		 */
		virtual QWidget *editWidget() = 0;

	protected:
		KopeteMessageManager *m_manager;
		ViewType m_type;
};

typedef QMap<KopeteProtocol*,KopeteChatWindow*> ProtocolMap;
typedef QMap<KopeteMessageManager*,KopeteView*>	ManagerMap;
typedef QMap<KopeteMessageManager*,KopeteEvent*> EventMap;
typedef QPtrList<KopeteChatWindow>		WindowList;

/**
 * Relates an actual chat to the means used to view it.
 */
class KopeteViewManager : public QObject
{
	Q_OBJECT
	public:
		/** This is a singleton class.  Call this method to get a pointer to 
		 * a KopeteViewManager.
		 */
		static KopeteViewManager *viewManager();

		~KopeteViewManager();

		/** 
		 * Return a view for the supplied KopeteMessageManager.  If one already
		 * exists, it will be returned, otherwise, a new view is created.
		 * @param manager The KopeteMessageManager we are viewing.
		 * @param foreignMessage Whether the message is inbound or outbound.
		 * @param type Specifies the type of view.
		 */
		KopeteView *view( KopeteMessageManager *, bool foreignMessage, KopeteView::ViewType type = KopeteView::Chat );

		/** 
		 * Make a view visible for the given KopeteMessageManager.
		 * @param manager The KopeteMessageManager we are viewing.
		 * @param type The type of view wanted.  If a view of any type for this KMM already exists, it will be made visible
		 */
		void launchWindow( KopeteMessageManager *, KopeteView::ViewType type );

		/**
		 * Provide access to the list of KopeteChatWindow the class maintains.
		 */
		WindowList windows;
		KopeteView *activeView() const { return m_activeView; };

	private:
		/**
	 	* Private constructor: we are a singleton
	 	*/
		KopeteViewManager();

		static KopeteViewManager *s_viewManager;

		ManagerMap m_managerMap;
		ProtocolMap m_protocolMap;
		EventMap m_eventMap;
		KopeteView *m_activeView;

		bool useQueue;
		bool raiseWindow;

		KopeteChatWindow *findWindow( KopeteProtocol * );

	public slots:
		/**
		 * Make a view visible and on top.
		 * @param manager The originating KopeteMessageManager.
		 * @param foreignMessage Whether the message is inbound or outbound.
		 */
		void readMessages( KopeteMessageManager*, bool foreignMessage );

		/**
		 * Called when a new message has been appended to the given
		 * KopeteMessageManager.  Procures a view for the message, and generates any notification events or displays messages, as appropriate.
		 * @param msg The new message
		 * @param manager The originating KopeteMessageManager
		 */
		void messageAppended( KopeteMessage &msg, KopeteMessageManager *manager);

	private slots:
		void slotViewDestroyed( KopeteView *);
		void slotChatWindowDestroyed( KopeteChatWindow * );
		void slotMessageManagerDestroyed( KopeteMessageManager * );
		void slotEventDeleted( KopeteEvent * );
		void slotPrefsChanged();
		void slotViewActivated( KopeteView * );
		
	signals:
		/**
		 * A new message is recieved, but not yet shown in the chatwindow
	 	 */
		void newMessageEvent(KopeteEvent *);

		/**
		 * A new view has been created
		 */
		void viewCreated( KopeteView * );

};

#endif
