/*
    kopetechatwindow.cpp - Chat Window

    Copyright (c) 2002      by Olivier Goffart       <ogoffart@tiscalinet.be>
    Copyright (C) 2002      by James Grant
    Copyright (c) 2002      by Stefan Gehn           <sgehn@gmx.net>
    Copyright (c) 2002-2003 by Martijn Klingens      <klingens@kde.org>

    Kopete    (c) 2002-2003 by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qfile.h>
#include <qtabwidget.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qtooltip.h>

#include <ktoolbar.h>
#include <klocale.h>
#include <kstdaction.h>
#include <kdockwidget.h>
#include <kaction.h>
#include <kmenubar.h>
#include <khelpmenu.h>
#include <kconfig.h>
#include <kglobal.h>
#include <kpopupmenu.h>
#include <kstandarddirs.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <kwin.h>
#include <ktempfile.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <ksqueezedtextlabel.h>
#include <kpushbutton.h>

#include "kopetemessage.h"
#include "kopeteemoticons.h"
#include "kopeteprotocol.h"
#include "kopeteplugin.h"
#include "kopeteprefs.h"
#include "pluginloader.h"
#include "kopetestdaction.h"
#include "kopetemetacontact.h"
#include "preferencesdialog.h"

#include "kopetechatwindow.h"
#include "kopetemessagemanager.h"
#include "chatview.h"

KopeteChatWindow::KopeteChatWindow(QWidget *parent, const char* name) : KMainWindow(parent, name)
{
	m_activeView = 0L;
	backgroundFile = 0L;
	updateBg = true;
	initActions();
	readOptions();
	m_tabBar = 0L;
	vBox = new QVBox( this );
	vBox->setLineWidth( 0 );
	setCentralWidget( vBox );

	mainArea = new QFrame( vBox );
	mainArea->setLineWidth( 0 );
	mainArea->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
	mainLayout = new QVBoxLayout( mainArea );

	QWidget *containerWidget = new QWidget(vBox);
	containerWidget->setSizePolicy( QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum) );

	//Layout for status label and send button
	QHBoxLayout *h = new QHBoxLayout( containerWidget, 4, 4 );

	//Status label
	m_status = new KSqueezedTextLabel( i18n("Ready."), containerWidget );
	m_status->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
	QToolTip::add( m_status, m_status->text() );
	h->addWidget( m_status, 0, Qt::AlignLeft | Qt::AlignVCenter );

	//Send Button
	m_button_send = new KPushButton( i18n("Send"), containerWidget );
	m_button_send->setEnabled( false );
	h->addWidget(m_button_send, 0, Qt::AlignRight | Qt::AlignVCenter);

	connect( m_button_send, SIGNAL( pressed() ), this, SLOT( slotSendMessage() ) );

	KopeteViewManager::viewManager()->windows.append( this );

	kdDebug( 14010 ) << k_funcinfo << "Open Windows: " << KopeteViewManager::viewManager()->windows.count() << endl;
}

KopeteChatWindow::~KopeteChatWindow()
{
	kdDebug(14010) << k_funcinfo << endl;
	emit( closing(this) );
	saveOptions();

	if( backgroundFile != 0L )
	{
		backgroundFile->close();
		backgroundFile->unlink();
	}
}

ChatView *KopeteChatWindow::activeView()
{
	return m_activeView;
}

void KopeteChatWindow::initActions(void)
{
	KActionCollection *coll = actionCollection();

	chatSend = new KAction( i18n( "&Send Message" ), QString::fromLatin1( "mail_send" ), 0,
		this, SLOT( slotSendMessage() ), coll, "chat_send" );
	//Default to "send" shortcut as used by KMail and KNode
	chatSend->setShortcut( QKeySequence(CTRL + Key_Return) );
	chatSend->setEnabled( false );

	chatSave = KStdAction::save ( this, SLOT(slotChatSave()), coll );
	chatPrint = KStdAction::print ( this, SLOT(slotChatPrint()), coll );
	chatCloseAll = KStdAction::quit ( this, SLOT(close()), coll );

	tabClose = KStdAction::close ( this, SLOT(slotChatClosed()), coll, "tabs_close" );
	tabClose->setIcon( QString::fromLatin1("tab_remove") );
	tabClose->setText( i18n("&Close Chat") );

	tabLeft = new KAction( i18n( "&Previous Chat" ), QString::fromLatin1( "1leftarrow" ), 0,
		this, SLOT( slotPreviousTab() ), coll, "tabs_left" );
	tabLeft->setShortcut( QKeySequence( CTRL + Key_PageDown) );
	tabLeft->setEnabled( false );

	tabRight = new KAction( i18n( "&Next Chat" ), QString::fromLatin1( "1rightarrow" ), 0,
		this, SLOT( slotNextTab() ), coll, "tabs_right" );
	tabRight->setShortcut( QKeySequence( CTRL + Key_PageUp ) );
	tabRight->setEnabled( false );

	tabDetach = new KAction( i18n( "&Detach Chat" ), QString::fromLatin1( "tab_breakoff" ), 0,
		this, SLOT( slotDetachChat() ), coll, "tabs_detach" );
	tabDetach->setEnabled( false );

	actionDetachMenu = new KActionMenu( i18n( "&Move Tab To Window..." ), QString::fromLatin1( "tab_breakoff" ), coll, "tabs_detachmove" );
	actionDetachMenu->setDelayed( false );

	connect ( actionDetachMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(slotPrepareDetachMenu()) );
	connect ( actionDetachMenu->popupMenu(), SIGNAL(activated(int)), this, SLOT(slotDetachChat(int)) );

	actionTabPlacementMenu = new KActionMenu( i18n( "&Tab Placement" ), coll, "tabs_placement" );
	connect ( actionTabPlacementMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(slotPreparePlacementMenu()) );
	connect ( actionTabPlacementMenu->popupMenu(), SIGNAL(activated(int)), this, SLOT(slotPlaceTabs(int)) );

	tabDetach->setShortcut( QKeySequence(CTRL + SHIFT + Key_B) );

	editCut = KStdAction::cut( this, SLOT(slotCut()), coll);
	editCopy = KStdAction::copy( this, SLOT(slotCopy()), coll);
	editPaste = KStdAction::paste( this, SLOT(slotPaste()), coll);

	editFont = new KAction( i18n( "&Set Font" ), QString::fromLatin1( "charset" ), 0,
		this, SLOT( slotSetFont() ), coll, "format_font" );
	editColor = new KAction( i18n( "Set Text &Color" ), QString::fromLatin1( "pencil" ), 0,
		this, SLOT( slotSetFgColor() ), coll, "format_color" );
	editBgColor = new KAction( i18n( "Set &Background Color" ), QString::fromLatin1( "fill" ), 0,
		this, SLOT( slotSetBgColor() ), coll, "format_bgcolor" );

	historyUp = new KAction( i18n( "Previous History" ), QString::null, 0,
		this, SLOT( slotHistoryUp() ), coll, "history_up" );
	historyUp->setShortcut( QKeySequence(CTRL + Key_Up) );

	historyDown = new KAction( i18n( "Next History" ), QString::null, 0,
		this, SLOT( slotHistoryDown() ), coll, "history_down" );
	historyDown->setShortcut( QKeySequence(CTRL + Key_Down) );

	// THIS CODE IS SPECIFIC TO KDE < 3.1
	// maybe #if KDE_VERSION < 310 around it?
	viewMenuBar = KStdAction::showMenubar( this, SLOT(slotViewMenuBar()), coll );
	viewToolBar = KStdAction::showToolbar( this, SLOT(slotViewToolBar()), coll );

	viewTabToolBar = KStdAction::showToolbar( this, SLOT(slotViewTabToolBar()), coll );
	viewTabToolBar->setText( i18n( "Show Ta&b Toolbar" ) );

	// END OF KDE < 3.1 SPECIFIC CODE

	membersLeft = new KToggleAction( i18n( "Show To Left of Chat Area" ), QString::null, 0,
		this, SLOT( slotViewMembersLeft() ), coll, "options_membersleft" );
	membersRight = new KToggleAction( i18n( "Show To Right of Chat Area" ), QString::null, 0,
		this, SLOT( slotViewMembersRight() ), coll, "options_membersright" );
	membersNone = new KToggleAction( i18n( "Hide" ), QString::null, 0,
		this, SLOT( slotViewMembersNone() ), coll, "options_membersnone" );
	membersNone->setChecked( true );

	actionSmileyMenu = new KActionMenu( i18n( "Add Smiley" ), QString::fromLatin1( "emoticon" ), coll, "format_smiley" );
	actionSmileyMenu->setDelayed( false );

	connect ( actionSmileyMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(slotPrepareSmileyMenu()) );
	connect ( actionSmileyMenu->popupMenu(), SIGNAL(activated(int)), this, SLOT(slotSmileyActivated(int)) );

	actionActionMenu = new KActionMenu(i18n("&Actions"), coll, "actions_menu" );
	connect ( actionActionMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(slotPrepareActionMenu()) );
//	connect ( actionActionMenu->popupMenu(), SIGNAL(activated(int)), this, SLOT(slotActionActivated(int)) );

	actionContactMenu = new KActionMenu(i18n("Co&ntacts"), coll, "contacts_menu" );
	connect ( actionContactMenu->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(slotPrepareContactMenu()) );

	// add configure key bindings menu item
	KStdAction::keyBindings(this, SLOT(slotConfKeys()), coll);
	KStdAction::configureToolbars(this, SLOT(slotConfToolbar()), coll);
	KStdAction::preferences( PreferencesDialog::preferencesDialog() , SLOT( show() ), coll );

	createGUI( QString::fromLatin1( "kopetechatwindow.rc" ) );
}

void KopeteChatWindow::setSendEnabled( bool enabled )
{
	chatSend->setEnabled( enabled );
	m_button_send->setEnabled( enabled );
}

void KopeteChatWindow::updateMembersActions()
{
	if( m_activeView )
	{
		switch ( m_activeView->membersListPosition() )
		{
			case KDockWidget::DockLeft:
				membersRight->setChecked( false );
				membersNone->setChecked( false );
				membersLeft->setChecked( true );
				break;
			case KDockWidget::DockRight:
				membersLeft->setChecked( false );
				membersNone->setChecked( false );
				membersRight->setChecked( true );
				break;
			default:
				membersLeft->setChecked( false );
				membersRight->setChecked( false );
				membersNone->setChecked( true );
				break;
		}
	}
}

void KopeteChatWindow::slotViewMembersLeft()
{
	m_activeView->placeMembersList( KDockWidget::DockLeft );
	updateMembersActions();
}

void KopeteChatWindow::slotViewMembersRight()
{
	m_activeView->placeMembersList( KDockWidget::DockRight );
	updateMembersActions();
}

void KopeteChatWindow::slotViewMembersNone()
{
	m_activeView->placeMembersList( KDockWidget::DockNone );
	updateMembersActions();
}

void KopeteChatWindow::slotHistoryUp()
{
	if( m_activeView )
		m_activeView->historyUp();
}

void KopeteChatWindow::slotHistoryDown()
{
	if( m_activeView )
		m_activeView->historyDown();
}

void KopeteChatWindow::slotCut()
{
	m_activeView->cut();
}

void KopeteChatWindow::slotCopy()
{
	m_activeView->copy();
}

void KopeteChatWindow::slotPaste()
{
	m_activeView->paste();
}

void KopeteChatWindow::slotSetBgColor()
{
	m_activeView->setBgColor();
}

void KopeteChatWindow::slotSetFgColor()
{
	m_activeView->setFgColor();
}

void KopeteChatWindow::slotSetFont()
{
	m_activeView->setFont();
}

ChatView *KopeteChatWindow::addChatView( KopeteMessageManager *manager )
{
	kdDebug(14010) << k_funcinfo << "Adding a new chat view" << endl;

	protocolIcon = UserIcon( manager->protocol()->protocolIcon() );

	//Save options so we can duplicate its appearance
	if( m_activeView )
		m_activeView->saveOptions();

	ChatView *newView;
	if( chatViewList.count() >= 1 )
	{
		newView = new ChatView ( manager, this, protocolIcon, 0L );
 	}
	else
	{
		newView = new ChatView ( manager, this, protocolIcon, mainArea );
		newView->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
		mainLayout->addWidget( newView );
		newView->show();
		m_activeView = newView;
	}

	chatViewList.append( newView );

	connect( newView, SIGNAL(captionChanged( bool)), this, SLOT(slotSetCaption(bool)) );

	return newView;
}


void KopeteChatWindow::setStatus(const QString &text)
{
	m_status->setText(text);
	if ( text != QToolTip::textFor( m_status ) )
		QToolTip::add( m_status, text );
}

void KopeteChatWindow::createTabBar()
{
	if( !m_tabBar )
	{
		m_tabBar = new QTabWidget( mainArea );
		m_tabBar->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
		mainLayout->addWidget( m_tabBar );
		m_tabBar->show();
		connect ( m_tabBar, SIGNAL(currentChanged(QWidget *)), this, SLOT(setActiveView(QWidget *)) );

		if( m_activeView )
		{
			m_activeView->reparent( m_tabBar, 0, QPoint(), true );
			m_tabBar->addTab( m_activeView, protocolIcon, m_activeView->caption() );
			m_activeView->setTabBar( m_tabBar );
		}

		actionTabPlacementMenu->setEnabled( true );
		KGlobal::config()->setGroup( QString::fromLatin1("ChatWindowSettings") );
		int tabPosition = KGlobal::config()->readNumEntry( QString::fromLatin1("Tab Placement") , 0 );
		slotPlaceTabs( tabPosition );
	}
}

void KopeteChatWindow::deleteTabBar()
{
	if( m_tabBar )
	{
		disconnect ( m_tabBar, SIGNAL(currentChanged(QWidget *)), this, SLOT(setActiveView(QWidget *)) );
		m_activeView->reparent( mainArea, 0, QPoint(), true );
		mainLayout->addWidget( m_activeView );
		m_activeView->setTabBar( 0L );
		m_tabBar->deleteLater();
		m_tabBar = 0L;
		actionTabPlacementMenu->setEnabled( false );
	}
}

void KopeteChatWindow::addChatView( ChatView* newView )
{
	createTabBar();
	newView->reparent( m_tabBar, 0, QPoint(), true );
	newView->setMainWindow( this );
	newView->setTabBar( 0L );
	newView->setCaption( newView->msgManager()->chatName(), false );
	chatViewList.append( newView );
	setActiveView( newView );
	setCaption( newView->caption() );
	checkDetachEnable();

	connect( newView, SIGNAL(captionChanged( bool)), this, SLOT(slotSetCaption(bool)) );
}

void KopeteChatWindow::checkDetachEnable()
{
	if( chatViewList.count() > 1 )
	{
		tabDetach->setEnabled( true );
		tabLeft->setEnabled( true );
		tabRight->setEnabled( true );
		actionTabPlacementMenu->setEnabled( true );
	}
	else
	{
		tabDetach->setEnabled( false );
		tabLeft->setEnabled( false );
		tabRight->setEnabled( false );
		actionTabPlacementMenu->setEnabled( false );
	}

	if( KopeteViewManager::viewManager()->windows.count() > 1 )
		actionDetachMenu->setEnabled( true );
	else
		actionDetachMenu->setEnabled( false );
}

void KopeteChatWindow::slotDetachChat( int newWindowIndex )
{
	KopeteChatWindow *newWindow = 0L;

	ChatView *detachedView = m_activeView;
	chatViewList.remove( detachedView );
	disconnect( detachedView, SIGNAL(captionChanged( bool)), this, SLOT(slotSetCaption(bool)) );

	if( m_tabBar )
	{
		int curPage = m_tabBar->currentPageIndex();
		if( curPage > 0 )
			m_tabBar->setCurrentPage( curPage - 1 );
		else
			m_tabBar->setCurrentPage( curPage + 1 );
		m_tabBar->removePage( detachedView );
	}

	if( newWindowIndex == -1 )
		newWindow = new KopeteChatWindow();
	else
		newWindow = KopeteViewManager::viewManager()->windows.at( newWindowIndex );

	newWindow->show();
	newWindow->raise();
	newWindow->addChatView( detachedView );

	checkDetachEnable();

	if( chatViewList.count() == 0 )
		delete this;
	else if( chatViewList.count() == 1)
		deleteTabBar();
}

void KopeteChatWindow::slotPreviousTab()
{
	int curPage = m_tabBar->currentPageIndex();
	if( curPage > 0 )
		m_tabBar->setCurrentPage( curPage - 1 );
	else
		m_tabBar->setCurrentPage( m_tabBar->count() - 1 );
}

void KopeteChatWindow::slotNextTab()
{
	int curPage = m_tabBar->currentPageIndex();
	if( curPage == ( m_tabBar->count() - 1 ) )
		m_tabBar->setCurrentPage( 0 );
	else
		m_tabBar->setCurrentPage( curPage + 1 );
}

void KopeteChatWindow::slotSetCaption( bool active )
{
	if( active && m_activeView )
	{
		setCaption( m_activeView->caption(), false );
	}
}

void KopeteChatWindow::updateBackground( const QPixmap &pm )
{
	if( updateBg )
	{
		updateBg = false;
		if( backgroundFile != 0L )
		{
			backgroundFile->close();
			backgroundFile->unlink();
		}

		backgroundFile = new KTempFile( QString::null, QString::fromLatin1( ".bmp" ) );
		pm.save( backgroundFile->name(), "BMP" );
		QTimer::singleShot( 100, this, SLOT( slotEnableUpdateBg() ) );
	}
}

void KopeteChatWindow::setActiveView( QWidget* active )
{
	kdDebug(14010) << k_funcinfo << endl;

	if( m_activeView )
		m_activeView->setActive( false );

	//Get the view switched to
	m_activeView = static_cast<ChatView*>(active);

	protocolIcon = UserIcon( m_activeView->msgManager()->protocol()->protocolIcon() );

	if( m_tabBar && !m_activeView->docked() )
		dockView( m_activeView );

	//Tell it it is active
	m_activeView->setActive( true );

	//Raise the tab
	if( m_tabBar )
		m_tabBar->showPage( m_activeView );

	setCaption( m_activeView->caption() );
	setStatus( m_activeView->status() );
	m_activeView->setFocus();

	//Update icons to match
	KWin::setIcons( winId(), protocolIcon, protocolIcon );

	//Update chat members actions
	updateMembersActions();
}

void KopeteChatWindow::dockView( ChatView *view )
{
	if( chatViewList.count() > 1 )
	{
		if( !m_tabBar )
			createTabBar();

		kdDebug(14010) << k_funcinfo << "Adding the tab" << endl;
		view->reparent( m_tabBar, 0, QPoint(), true );
		m_tabBar->addTab( view, protocolIcon, view->caption() );
		view->setTabBar( m_tabBar );
		m_tabBar->showPage( m_activeView );
		checkDetachEnable();
	}
}

void KopeteChatWindow::slotChatClosed()
{
	m_activeView->close();
}

void KopeteChatWindow::closeChat( ChatView *closedView )
{
	kdDebug(14010) << k_funcinfo << endl;
	if( closedView == m_activeView )
		m_activeView = 0L;

	chatViewList.remove( closedView );

	if( chatViewList.isEmpty() )
		delete this;
	else if( m_tabBar)
	{
		m_tabBar->removePage( closedView );
		checkDetachEnable();
	}

	if( m_tabBar && chatViewList.count() == 1 )
		deleteTabBar();
}

void KopeteChatWindow::slotPrepareDetachMenu(void)
{
	QPopupMenu *detachMenu = actionDetachMenu->popupMenu();
	detachMenu->clear();
	WindowList windowList = KopeteViewManager::viewManager()->windows;

	int id = 0;
	for ( KopeteChatWindow *window = windowList.first(); window; window = windowList.next() )
	{
		if( window != this )
			detachMenu->insertItem( window->caption(), id );
		id++;
	}
}

void KopeteChatWindow::slotSendMessage(void)
{
	if( m_activeView )
		m_activeView->sendMessage();
}

void KopeteChatWindow::slotPrepareSmileyMenu(void)
{
	QPopupMenu *smileyMenu = actionSmileyMenu->popupMenu();

	smileyMenu->clear();

	QMap<QString, QString> list = KopeteEmoticons::emoticons()->emoticonAndPicList();

	for (QMap<QString, QString>::Iterator it = list.begin(); it != list.end(); ++it )
		smileyMenu->insertItem(QPixmap( it.data() ),it.key());
}

void KopeteChatWindow::slotPrepareContactMenu(void)
{
	QPopupMenu *contactsMenu = actionContactMenu->popupMenu();
	contactsMenu->clear();

	KopeteContact *contact;
	KopeteContactPtrList m_them = m_activeView->msgManager()->members();

	for ( contact = m_them.first(); contact != 0; contact = m_them.next() )
	{
		if( contact->metaContact() )
			contactsMenu->insertItem( UserIcon(contact->statusIcon()) , contact->metaContact()->displayName(), contact->popupMenu() );
		else
			contactsMenu->insertItem( UserIcon(contact->statusIcon()) , contact->displayName(), contact->popupMenu() );
	}
}

void KopeteChatWindow::slotPreparePlacementMenu(void)
{
	QPopupMenu *placementMenu = actionTabPlacementMenu->popupMenu();
	placementMenu->clear();

	placementMenu->insertItem( i18n("Top"), 0 );
	placementMenu->insertItem( i18n("Bottom"), 1 );
}

void KopeteChatWindow::slotPlaceTabs( int placement )
{
	if( m_tabBar )
	{
		kdDebug(14010) << k_funcinfo << "Placement:" << placement << endl;
		if( placement == 0 )
			m_tabBar->setTabPosition( QTabWidget::Top );
		else
			m_tabBar->setTabPosition( QTabWidget::Bottom );

		saveOptions();
	}
}

void KopeteChatWindow::slotPrepareActionMenu(void)
{
	QPopupMenu *actionsMenu = actionActionMenu->popupMenu();

	actionsMenu->clear();

	QPtrList<KopetePlugin> ps = LibraryLoader::pluginLoader()->plugins();
	bool actions = false;

	for( KopetePlugin *p = ps.first() ; p ; p = ps.next() )
	{
		KActionCollection *customActions = p->customChatActions( m_activeView->msgManager() );
		if( customActions )
		{
			kdDebug(14010) << k_funcinfo << "Found custom Actions defined by Plugins" << endl;
			actions = true;
			for(unsigned int i = 0; i < customActions->count(); i++)
			{
				customActions->action(i)->plug( actionsMenu );
			}
		}
	}

	if ( !actions )
	{
		kdDebug(14010) << k_funcinfo << "No Action defined by any Plugin" << endl;
		int id = actionsMenu->insertItem( i18n("No Action defined by any Plugin") );
		actionsMenu->setItemEnabled(id, false);
	}
}

void KopeteChatWindow::readOptions()
{
	// load and apply config file settings affecting the appearance of the UI
	kdDebug(14010) << k_funcinfo << endl;
	KConfig *config = KGlobal::config();
	applyMainWindowSettings( config, QString::fromLatin1( "KopeteChatWindow" ) );
}

void KopeteChatWindow::saveOptions()
{
	kdDebug(14010) << k_funcinfo << endl;

	KConfig *config = KGlobal::config();

	// saves menubar,toolbar and statusbar setting
	saveMainWindowSettings( config, QString::fromLatin1( "KopeteChatWindow" ) );

	if( m_tabBar )
	{
		config->setGroup( QString::fromLatin1("ChatWindowSettings") );
		config->writeEntry ( QString::fromLatin1("Tab Placement"), m_tabBar->tabPosition() );
	}

	config->sync();
}

void KopeteChatWindow::slotChatSave()
{
//	kdDebug(14010) << "KopeteChatWindow::slotChatSave()" << endl;

	QString fileName = KFileDialog::getSaveFileName ( QString::null, QString::fromLatin1( "text/html" ),
		this, i18n( "Save Conversation" ) );

	if ( fileName.isEmpty() )
		return;

	QFile file ( fileName );
	if ( file.open(IO_WriteOnly) )
	{
		QTextStream stream ( &file );
		stream << m_activeView->viewsText(); // text() is full-featured html
		file.close(); // maybe unneeded but I like to close opened files ;)
	}
	else
	{
		KMessageBox::error( 0, //No parent
			i18n("Could not open %1 for writing").arg(fileName), // Message
			i18n("Error while saving") ); //Caption
	}
}

void KopeteChatWindow::windowActivationChange( bool )
{
	if( isActiveWindow() && m_activeView )
		m_activeView->setActive( true );
}

// KDE < 3.1 SPECIFIC
void KopeteChatWindow::slotChatPrint()
{
	m_activeView->print();
}

void KopeteChatWindow::slotViewToolBar()
{
	if(toolBar()->isVisible())
		toolBar()->hide();
	else
		toolBar()->show();
}

void KopeteChatWindow::slotViewMenuBar()
{
	if( !menuBar()->isHidden() )
		menuBar()->hide();
	else
		menuBar()->show();
}
 
void KopeteChatWindow::slotViewTabToolBar()
{
	if ( toolBar( "tabToolBar" )->isHidden() )
		toolBar( "tabToolBar" )->show();
	else
		toolBar( "tabToolBar" )->hide();
}

// END KDE < 3.1 SPECIFIC

void KopeteChatWindow::slotSmileyActivated(int sm)
{
	// FIXME: it should read from the emoticonlist instead of using the text
	m_activeView->addText( actionSmileyMenu->popupMenu()->text(sm) );
}

void KopeteChatWindow::closeEvent( QCloseEvent *e )
{
	close();
	e->ignore();
}

bool KopeteChatWindow::close(bool)
{
	bool retVal = true;
	if( chatViewList.count() > 0 )
	{
		while( ChatView *view = chatViewList.first() )
		{
			chatViewList.remove( view );
			retVal = retVal && view->close();
		}
	}
	return retVal;
}

void KopeteChatWindow::slotConfKeys()
{
	KKeyDialog::configureKeys(actionCollection(), xmlFile(), true, this);
}

void KopeteChatWindow::slotConfToolbar()
{
	saveMainWindowSettings(KGlobal::config(), QString::fromLatin1( "KopeteChatWindow" ));
	KEditToolbar *dlg = new KEditToolbar(actionCollection(), QString::fromLatin1("kopetechatwindow.rc") );
	if (dlg->exec())
	{
		createGUI( QString::fromLatin1("kopetechatwindow.rc") );
		applyMainWindowSettings(KGlobal::config(), QString::fromLatin1( "KopeteChatWindow" ));
	}
	delete dlg;
}

bool KopeteChatWindow::queryExit()
{
//	 never quit kopete
	return false;
}

#include "kopetechatwindow.moc"

// vim: set noet ts=4 sts=4 sw=4:

